@@ -1,26 +1,25 @@
 import logging
 import numexpr
 import os
-import flarestack.core.astro
 import numpy as np
 import scipy.interpolate
 from scipy import sparse
 import pickle
 from flarestack.shared import acceptance_path, llh_energy_hash_pickles, \
-    SoB_spline_path, bkg_spline_path
+    SoB_spline_path, default_gamma_precision, default_smoothing_order
 from flarestack.core.time_pdf import TimePDF, read_t_pdf_dict
-from flarestack.utils.make_SoB_splines import load_spline, \
-    load_bkg_spatial_spline
+from flarestack.utils.make_SoB_splines import load_spline
 from flarestack.core.energy_pdf import EnergyPDF, read_e_pdf_dict
 from flarestack.core.spatial_pdf import SpatialPDF
 from flarestack.utils.create_acceptance_functions import dec_range,\
     make_acceptance_season
-from flarestack.icecube_utils.dataset_loader import data_loader
 from flarestack.utils.make_SoB_splines import create_2d_ratio_hist, \
-    make_2d_spline_from_hist, \
-    make_background_spline, make_individual_spline_set
+    make_2d_spline_from_hist, make_individual_spline_set
+
+logger = logging.getLogger(__name__)
 
 spatial_mask_threshold = 1e-21
+default_spacial_box_width = 5.  # box width around source positions in degrees
 
 
 def read_llh_dict(llh_dict):
@@ -42,7 +41,7 @@ def read_llh_dict(llh_dict):
     for (old_key, new_key) in maps:
 
         if old_key in list(llh_dict.keys()):
-            logging.warning("Deprecated llh_dict key'{0}' was used. Please use '{1}' in future.".format(
+            logger.warning("Deprecated llh_dict key'{0}' was used. Please use '{1}' in future.".format(
                 old_key, new_key))
             llh_dict[new_key] = llh_dict[old_key]
 
@@ -61,7 +60,7 @@ def read_llh_dict(llh_dict):
         llh_dict["llh_spatial_pdf"] = {}
 
     if "llh_bkg_time_pdf" not in llh_dict.keys():
-        logging.warning("No 'llh_bkg_time_pdf' was specified. A 'steady' pdf will be assumed.")
+        logger.warning("No 'llh_bkg_time_pdf' was specified. A 'steady' pdf will be assumed.")
         llh_dict["llh_bkg_time_pdf"] = {
             "time_pdf_name": "steady"
         }
@@ -79,6 +78,7 @@ class LLH(object):
         self.sources = sources
         self.llh_dict = llh_dict
         self.spatial_pdf = SpatialPDF(llh_dict["llh_spatial_pdf"], season)
+        self.spatial_box_width = llh_dict.get('spatial_box_width', default_spacial_box_width)
 
         try:
             time_dict = llh_dict["llh_sig_time_pdf"]
@@ -262,7 +262,7 @@ class LLH(object):
         for source in sources:
 
             # Sets half width of spatial box
-            width = np.deg2rad(5.)
+            width = np.deg2rad(self.spatial_box_width)
 
             # Sets a declination band 5 degrees above and below the source
             min_dec = max(-np.pi / 2., source['dec_rad'] - width)
@@ -477,6 +477,22 @@ class FixedEnergyLLH(LLH):
                            "dictionary to the LLH dictionary, and try "
                            "again.")
 
+        # defines the order of the splines used in the building of the energy PDF
+        self.smoothing_order = None
+        smoothing_order = llh_dict.get("smoothing_order", 'flarestack')
+        if isinstance(smoothing_order, str):
+            self.smoothing_order = default_smoothing_order[smoothing_order]
+        else:
+            self.smoothing_order = smoothing_order
+
+        # used to construct the support points in gamma when the energy PDF is built
+        self.precision = None
+        precision = llh_dict.get('gamma_precision', 'flarestack')
+        if isinstance(precision, str):
+            self.precision = default_gamma_precision[precision]
+        else:
+            self.precision = precision
+
         LLH.__init__(self, season, sources, llh_dict)
 
     def create_energy_functions(self):
@@ -495,7 +511,7 @@ class FixedEnergyLLH(LLH):
         if not os.path.isfile(acc_path):
             self.create_acceptance_function(acc_path)
 
-        logging.debug("Loading from {0}".format(acc_path))
+        logger.debug("Loading from {0}".format(acc_path))
 
         with open(acc_path, "rb") as f:
             [dec_vals, acc] = pickle.load(f)
@@ -511,13 +527,13 @@ class FixedEnergyLLH(LLH):
         if not os.path.isfile(SoB_path):
             self.create_energy_weighting_function(SoB_path)
 
-        logging.debug("Loading from {0}".format(SoB_path))
+        logger.debug("Loading from {0}".format(SoB_path))
 
         with open(SoB_path, "rb") as f:
             [dec_vals, ratio_hist] = pickle.load(f)
 
         spline = make_2d_spline_from_hist(np.array(ratio_hist), dec_vals,
-                                          self.season.log_e_bins)
+                                          self.season.log_e_bins, self.smoothing_order)
 
         def energy_weight_f(event, params=None):
             return np.exp(spline(event["logE"], event["sinDec"], grid=False))
@@ -525,7 +541,7 @@ class FixedEnergyLLH(LLH):
         return acc_f, energy_weight_f
 
     def create_acceptance_function(self, acc_path):
-        logging.info("Building acceptance functions in sin(dec) bins "
+        logger.info("Building acceptance functions in sin(dec) bins "
                      "(with fixed energy weighting)")
 
         mc = self.season.get_pseudo_mc()
@@ -557,7 +573,7 @@ class FixedEnergyLLH(LLH):
         except OSError:
             pass
 
-        logging.info("Saving to {0}".format(acc_path))
+        logger.info("Saving to {0}".format(acc_path))
 
         with open(acc_path, "wb") as f:
             pickle.dump([dec_range, acc], f)
@@ -565,7 +581,7 @@ class FixedEnergyLLH(LLH):
         return f
 
     def create_energy_weighting_function(self, SoB_path):
-        logging.info("Building energy-weighting functions in sin(dec) vs log E bins "
+        logger.info("Building energy-weighting functions in sin(dec) vs log E bins "
                      "(with fixed energy weighting)")
 
         # dec_range = self.season["sinDec bins"]
@@ -583,7 +599,7 @@ class FixedEnergyLLH(LLH):
         except OSError:
             pass
 
-        logging.info("Saving to {0}".format(SoB_path))
+        logger.info("Saving to {0}".format(SoB_path))
 
         with open(SoB_path, "wb") as f:
             pickle.dump([dec_range, ratio_hist], f)
@@ -673,11 +689,18 @@ class StandardLLH(FixedEnergyLLH):
         self.energy_bins = np.linspace(1., 10., 40 + 1)
 
         # Sets precision for energy SoB
-        self.precision = .1
+        # self.precision = .1
 
-        self.SoB_spline_2Ds = load_spline(self.season)
+        self.SoB_spline_2Ds = load_spline(self.season,
+                                          smoothing_order=self.smoothing_order,
+                                          gamma_precision=self.precision)
 
-        logging.debug("Loaded {0} splines.".format(len(self.SoB_spline_2Ds)))
+        if self.SoB_spline_2Ds:
+            logger.debug("Loaded {0} splines.".format(len(self.SoB_spline_2Ds)))
+        else:
+            logger.warning("Didn\'t load SoB splines!!!")
+
+        logger.debug("Loaded {0} splines.".format(len(self.SoB_spline_2Ds)))
 
         self.acceptance_f = self.create_acceptance_function()
         self.acceptance = self.new_acceptance
@@ -702,7 +725,7 @@ class StandardLLH(FixedEnergyLLH):
         :return: Acceptance function, energy_weighting_function
         """
 
-        SoB_path = SoB_spline_path(self.season)
+        SoB_path = SoB_spline_path(self.season, smoothing_order=self.smoothing_order, gamma_precision=self.precision)
         acc_path = acceptance_path(self.season)
 
         # Set up acceptance function, creating values if they have not been
@@ -711,7 +734,7 @@ class StandardLLH(FixedEnergyLLH):
         if not os.path.isfile(acc_path):
             make_acceptance_season(self.season, acc_path)
 
-        logging.debug("Loading from {0}".format(acc_path))
+        logger.debug("Loading from {0}".format(acc_path))
 
         with open(acc_path, "rb") as f:
             [dec_bins, gamma_bins, acc] = pickle.load(f)
@@ -727,7 +750,9 @@ class StandardLLH(FixedEnergyLLH):
         # Checks if energy weighting functions have been created
 
         if not os.path.isfile(SoB_path):
-            make_individual_spline_set(self.season, SoB_path)
+            make_individual_spline_set(self.season, SoB_path,
+                                       smoothing_order=self.smoothing_order,
+                                       gamma_precision=self.precision)
 
         return acc_f, None
 
@@ -822,9 +847,7 @@ class StandardLLH(FixedEnergyLLH):
                     SoB_pdf = lambda x: self.signal_pdf(source, x) / \
                                         self.background_pdf(source, x)
 
-                    spatial_cache = pull_corrector.create_spatial_cache(
-                        coincident_data, SoB_pdf
-                    )
+                    spatial_cache = pull_corrector.create_spatial_cache(coincident_data, SoB_pdf)
 
                     SoB_spacetime.append(spatial_cache)
 
@@ -930,8 +953,11 @@ class StandardLLH(FixedEnergyLLH):
         energy_SoB_cache = dict()
 
         for gamma in list(self.SoB_spline_2Ds.keys()):
-            energy_SoB_cache[gamma] = self.SoB_spline_2Ds[gamma].ev(
-                cut_data["logE"], cut_data["sinDec"])
+            try:
+                energy_SoB_cache[gamma] = self.SoB_spline_2Ds[gamma].ev(
+                    cut_data["logE"], cut_data["sinDec"])
+            except:  # this is in case the splines were produced using the RegularGridInterpolator
+                energy_SoB_cache[gamma] = self.SoB_spline_2Ds[gamma]((cut_data["logE"], cut_data["sinDec"]))
 
         return energy_SoB_cache
 
@@ -1043,9 +1069,7 @@ class StandardOverlappingLLH(StandardLLH):
                 for i, source in enumerate(self.sources)], axis=0
             ) / np.sum(season_weight(gamma))
 
-        SoB_spacetime = pull_corrector.create_spatial_cache(
-            coincident_data, joint_SoB
-        )
+        SoB_spacetime = pull_corrector.create_spatial_cache(coincident_data, joint_SoB)
 
         kwargs["n_coincident"] = np.sum(~assumed_background_mask)
         kwargs["SoB_spacetime_cache"] = SoB_spacetime
@@ -1171,9 +1195,7 @@ class StandardMatrixLLH(StandardOverlappingLLH):
 
             return return_value
 
-        SoB_spacetime = pull_corrector.create_spatial_cache(
-            coincident_data, joint_SoB
-        )
+        SoB_spacetime = pull_corrector.create_spatial_cache(coincident_data, joint_SoB)
 
         kwargs["n_coincident"] = np.sum(coincident_nu_mask)
         kwargs["SoB_spacetime_cache"] = SoB_spacetime
