@@ -10,7 +10,7 @@ import matplotlib.pyplot as plt
 from flarestack.shared import name_pickle_output_dir, plot_output_dir, \
     k_to_flux, inj_dir_name, scale_shortener, flux_to_k
 from flarestack.core.ts_distributions import plot_background_ts_distribution, \
-    plot_fit_results
+    plot_fit_results, get_ts_fit_type
 from flarestack.utils.neutrino_astronomy import calculate_astronomy
 from flarestack.core.minimisation import MinimisationHandler
 from flarestack.utils.catalogue_loader import load_catalogue
@@ -43,10 +43,7 @@ class ResultsHandler(object):
 
         # Checks if the code should search for flares. By default, this is
         # not done.
-        # try:
-        #     self.flare = llh_kwargs["Flare Search?"]
-        # except KeyError:
-        #     self.flare = False
+        # self.flare = self.mh_name == "flare"
 
         # if self.flare:
         #     self.make_plots = self.flare_plots
@@ -74,8 +71,8 @@ class ResultsHandler(object):
         # elif self.negative_n_s:
         #     self.ts_type = "Negative n_s"
         # else:
-        self.ts_type = "Standard"
-        #
+        self.ts_type = get_ts_fit_type(rh_dict)
+
         # print "negative_ns", self.negative_n_s
 
         p0, bounds, names = MinimisationHandler.find_parameter_info(rh_dict)
@@ -312,14 +309,14 @@ class ResultsHandler(object):
 
         try:
             # if weights were not fitted, number of neutrinos is stored in just one parameter
-            if 'fit' not in self.mh_name:
-                # self.flux_to_ns = self.inj[self.scales[1]]["n_s"] / k_to_flux(float(x[1]))
+
+            if "n_s" in self.inj[self.scales[1]]:
+
                 self.flux_to_ns = self.inj[self.scales[1]]["n_s"] / k_to_flux(self.scales_float[1])
 
-            # if weights were fitted, there is one n_s for each fitted source
+            # if weights were fitted, or for cluster search, there is one n_s for each fitted source
             else:
                 sc_dict = self.inj[self.scales[1]]
-                # self.flux_to_ns = sum([sc_dict[k] for k in sc_dict if 'n_s' in str(k)]) / k_to_flux(float(x[1]))
                 self.flux_to_ns = sum([sc_dict[k] for k in sc_dict if 'n_s' in str(k)]) / k_to_flux(self.scales_float[1])
 
             logger.debug(f"Conversion ratio of flux to n_s: {self.flux_to_ns:.2f}")
@@ -491,6 +488,9 @@ class ResultsHandler(object):
 
                 self.make_plots(scale)
 
+        if len(np.where(np.array(y) < 0.95)[0]) < 2:
+            raise OverfluctuationError(f"Not enough points with overfluctuations under 95%, lower injection scale!")
+
         x = np.array(x_acc)
         self.overfluctuations[ts_val] = x, y, yerr
 
