@@ -10,7 +10,7 @@ from flarestack.core.time_pdf import TimePDF
 from flarestack.shared import name_pickle_output_dir, plot_output_dir, \
     analysis_pickle_path, limit_output_path, unblinding_output_path
 import pickle
-from flarestack.core.ts_distributions import plot_background_ts_distribution
+from flarestack.core.ts_distributions import plot_background_ts_distribution, get_ts_fit_type
 import matplotlib.pyplot as plt
 from flarestack.utils.catalogue_loader import load_catalogue
 
@@ -74,7 +74,7 @@ def create_unblinder(unblind_dict, mock_unblind=True, full_plots=False,
             unblind_dict["unblind_bool"] = True
             unblind_dict["mock_unblind_bool"] = mock_unblind
             unblind_dict["inj_dict"] = {}
-
+            self.background_median = np.nan
 
             if np.logical_and(not mock_unblind, not disable_warning):
                 self.check_unblind()
@@ -144,8 +144,8 @@ def create_unblinder(unblind_dict, mock_unblind=True, full_plots=False,
 
             # Quantify the TS value significance
             self.ts = np.array([self.res_dict["TS"]])[0]
-            self.ts_type = unblind_dict.get('ts_type', 'Standard')
             self.sigma = np.nan
+            self.ts_type = get_ts_fit_type(unblind_dict)
 
             logger.info("Test Statistic of: {0}".format(self.ts))
 
@@ -221,12 +221,12 @@ def create_unblinder(unblind_dict, mock_unblind=True, full_plots=False,
 
                     savepath = ul_dir + subdir + ".pdf"
 
-                    if self.ts < rh.bkg_median:
-                        logging.warning(f"Specified TS ({self.ts}) less than the background median ({rh.bkg_median}). "
-                                        f"There was thus a background n underfluctuation. "
+                    if self.ts < self.bkg_median:
+                        logging.warning(f"Specified TS ({self.ts}) less than the background median ({self.bkg_median}). "
+                                        f"There was thus a background underfluctuation. "
                                         f"Using the sensitivity for an upper limit.")
 
-                    ul, extrapolated, err = rh.find_overfluctuations(max(self.ts, rh.bkg_median), savepath)
+                    ul, extrapolated, err = rh.find_overfluctuations(max(self.ts, self.bkg_median), savepath)
 
                     flux_uls.append(ul)
 
@@ -347,6 +347,8 @@ def create_unblinder(unblind_dict, mock_unblind=True, full_plots=False,
                 self.sigma = plot_background_ts_distribution(
                     ts_array, self.output_file, self.ts_type, self.ts, self.mock_unblind)
 
+                self.background_median = np.median(ts_array)
+
             except (IOError, OSError):
 
                 try:
