@@ -1,29 +1,63 @@
 import logging
 import numpy as np
 import os
+import shutil
 import scipy.interpolate
 import pickle as Pickle
-from flarestack.shared import gamma_precision, SoB_spline_path, \
-    bkg_spline_path, dataset_plot_dir, get_base_sob_plot_dir
+from flarestack.shared import default_gamma_precision, SoB_spline_path, default_smoothing_order, \
+    bkg_spline_path, get_base_sob_plot_dir, flarestack_gamma_precision
 from flarestack.core.energy_pdf import PowerLaw
-from flarestack.icecube_utils.dataset_loader import data_loader
 import matplotlib.pyplot as plt
 
+environment_smoothing_key = 'FLARESATCK_SMOOTHING_ORDER'
+environment_precision_key = 'FLARESTACK_PRECISION'
+logger = logging.getLogger(__name__)
 energy_pdf = PowerLaw()
 
-def _around(value):
+
+def get_gamma_precision(precision=flarestack_gamma_precision):
+    """Returns the precision in gamma that is used.
+    Returns default value if the environment_precision_key is not present in th environ dictionary
+
+    :param precision: Specify which precision to use. Default to the standard precision.
+    Can also provide default of llh codes by name, either 'skylab' or 'flarestack'.
+    :return: Precision as a float
+    """
+    if isinstance(precision, float):
+        return precision
+
+    elif isinstance(precision, str):
+        if precision in default_gamma_precision:
+            return default_gamma_precision[precision]
+        else:
+            raise ValueError(f'No gamma precision value defined for {precision}!'
+                             f'Choose: {default_gamma_precision.keys()}')
+
+    else:
+        raise TypeError(f'Type {type(precision)} of {precision} not understood for variable gamma precision')
+
+
+def _around(value, precision=flarestack_gamma_precision):
     """Produces an array in which the precision of the value
     is rounded to the nearest integer. This is then multiplied
     by the precision, and the new value is returned.
 
-    :param value: value to be processed
+    :param precision: Specify which precision to use. Default to the standard precision.
+    Can also provide default of llh codes by name, either 'skylab' or 'flarestack'.
     :return: value after processed
     """
-    return np.around(float(value) / gamma_precision) * gamma_precision
+    return np.around(float(value) / get_gamma_precision(precision)) * get_gamma_precision(precision)
 
 
-gamma_points = np.arange(0.7, 4.3, gamma_precision)
-gamma_support_points = set([_around(i) for i in gamma_points])
+def get_gamma_support_points(precision=flarestack_gamma_precision):
+    """Return the gamma support points based on the gamma precision
+
+    :param precision: Specify which precision to use. Default to the standard precision.
+    Can also provide default of llh codes by name, either 'skylab' or 'flarestack'.
+    :return: Gamma support points
+    """
+    gamma_points = np.arange(0.7, 4.3, get_gamma_precision(precision=precision))
+    return set([_around(i, precision=precision) for i in gamma_points])
 
 
 def create_2d_hist(sin_dec, log_e, sin_dec_bins, log_e_bins, weights):
@@ -40,8 +74,12 @@ def create_2d_hist(sin_dec, log_e, sin_dec_bins, log_e_bins, weights):
     :return: Normalised histogram
     """
     # Produces the histogram
+    # rangee = tuple([(wB_i[0], wB_i[-1]) for wB_i in [log_e_bins, sin_dec_bins]])
+    # if np.all(weights == 1):
+    #     weights = None
+        # print('no weights')
     hist_2d, binedges = np.histogramdd(
-        (log_e, sin_dec), bins=(log_e_bins, sin_dec_bins), weights=weights)\
+        (log_e, sin_dec), bins=(log_e_bins, sin_dec_bins), weights=weights)  # , range=rangee, normed=True)
 
     # n_dimensions = hist_2d.ndim
 
@@ -125,9 +163,21 @@ def create_2d_ratio_hist(exp, mc, sin_dec_bins, log_e_bins, weight_function):
     norms = np.sum(sig_hist, axis=n_dimensions - 2)
     norms[norms == 0.] = 1.
     sig_hist /= norms
+    # bkg_norms = np.sum(bkg_hist, axis=tuple(range(n_dimensions - 1)))
+    # bkg_norms[bkg_norms == 0.] = 1
+    # bkg_hist /= bkg_norms
 
     ratio = np.ones_like(bkg_hist, dtype=np.float)
 
+    # wSd = sig_hist > 0
+    # wB_domain = bkg_hist > 0
+    # ratio[wSd & wB_domain] = (sig_hist[wSd & wB_domain] / bkg_hist[wSd & wB_domain])
+    # # values outside of the exp domain, but inside the MC one are mapped to
+    # # the most signal-like value
+    # if np.any(ratio > 1):
+    #     min_ratio = np.percentile(ratio[ratio > 1.], 99.)
+    #     np.copyto(ratio, min_ratio, where=wSd & ~wB_domain)
+
     for i, bkg_row in enumerate(bkg_hist.T):
         sig_row = sig_hist.T[i]
 
@@ -146,10 +196,15 @@ def create_2d_ratio_hist(exp, mc, sin_dec_bins, log_e_bins, weight_function):
     #
     # input("prompt")
 
+    # print('background      signal       ratio')
+    # for j, (b, s, r) in enumerate(zip(bkg_hist[:, 0], sig_hist[:, 0], ratio[:, 0])):
+    #     print(f'{j}: {b:.4f}      {s:.4f}     {r:.4f}')
+    # input('continue? ')
+
     return ratio
 
 
-def create_2d_ratio_spline(exp, mc, sin_dec_bins, log_e_bins, weight_function):
+def create_2d_ratio_spline(exp, mc, sin_dec_bins, log_e_bins, weight_function, smoothing_order):
     """Creates 2D histograms for both data and MC, in which the seasons
     are binned by Sin(Declination) and Log(Energy/GeV). Each histogram is
     normalised in Sin(Declination) bands. Then creates a histogram of the
@@ -173,27 +228,57 @@ def create_2d_ratio_spline(exp, mc, sin_dec_bins, log_e_bins, weight_function):
     ratio = create_2d_ratio_hist(exp, mc, sin_dec_bins, log_e_bins,
                                  weight_function)
 
-    spline = make_2d_spline_from_hist(ratio, sin_dec_bins, log_e_bins)
+    spline = make_2d_spline_from_hist(ratio, sin_dec_bins, log_e_bins, smoothing_order)
 
     return spline
 
 
-def make_2d_spline_from_hist(ratio, sin_dec_bins, log_e_bins):
+def make_2d_spline_from_hist(ratio, sin_dec_bins, log_e_bins, smoothing_order):
 
     # Sets bin centers, and order of spline (for x and y)
     sin_bin_center = (sin_dec_bins[:-1] + sin_dec_bins[1:]) / 2.
     log_e_bin_center = (log_e_bins[:-1] + log_e_bins[1:]) / 2.
-    order = 2
+
+    # the order of the splines defaults to 2
+    # default_order = 2
+    if not isinstance(smoothing_order, int):
+        smoothing_order = default_smoothing_order[smoothing_order]
+
+    # if the environment_smoothing_key is present in the environ dictionary use the corresponding value instead
+    # _order = os.environ.get(environment_smoothing_key, default_order)
+    # order = None if _order == 'None' else int(_order)
+
+    # when setting the emviron value to 'None', order will be None and no splines will be produced
+    if isinstance(smoothing_order, type(None)):
+        logger.warning(f'{environment_smoothing_key} is None! Not making splines!')
+        return
 
     # Fits a 2D spline function to the log of ratio array
-    # This is 2nd order in both dimensions
-    spline = scipy.interpolate.RectBivariateSpline(
-        log_e_bin_center, sin_bin_center, np.log(ratio),
-        kx=order, ky=order, s=0)
+    # If the splines are of order one the RegularGridInterpolator is used to match the SkyLab behavior
+    if smoothing_order == 1:
+
+        sin_bin_center[0], sin_bin_center[-1] = sin_dec_bins[0], sin_dec_bins[-1]
+        # log_e_bins[0], log_e_bins[-1] = log_e_bins[0], log_e_bins[-1]
+
+        binmids = (log_e_bin_center, sin_bin_center)
+
+        spline = scipy.interpolate.RegularGridInterpolator(
+            binmids, np.log(ratio),
+            method="linear",
+            bounds_error=False,
+            fill_value=0.)
+
+    # If the interpolating splines are of order greater than 1, use RectBivariateSpline
+    else:
+        # This is order-th order in both dimensions
+        spline = scipy.interpolate.RectBivariateSpline(
+            log_e_bin_center, sin_bin_center, np.log(ratio),
+            kx=smoothing_order, ky=smoothing_order, s=0)
+
     return spline
 
 
-def create_gamma_2d_ratio_spline(exp, mc, sin_dec_bins, log_e_bins, gamma):
+def create_gamma_2d_ratio_spline(exp, mc, sin_dec_bins, log_e_bins, gamma, smoothing_order):
     """Creates a 2D gamma ratio spline by creating a function that weights MC
     assuming a power law of spectral index gamma.
 
@@ -208,10 +293,10 @@ def create_gamma_2d_ratio_spline(exp, mc, sin_dec_bins, log_e_bins, gamma):
         return energy_pdf.weight_mc(sig_mc, gamma)
 
     return create_2d_ratio_spline(exp, mc, sin_dec_bins, log_e_bins,
-                                  weight_function)
+                                  weight_function, smoothing_order)
 
 
-def create_2d_splines(exp, mc, sin_dec_bins, log_e_bins):
+def create_2d_splines(exp, mc, sin_dec_bins, log_e_bins, **kwargs):
     """If gamma will not be fit, then calculates the Log(Signal/Background)
     2D PDF for the fixed value self.default_gamma. Fits a spline to each
     histogram, and saves the spline in a dictionary.
@@ -228,10 +313,18 @@ def create_2d_splines(exp, mc, sin_dec_bins, log_e_bins):
     :return: Dictionary of 2D Log(Signal/Background) splines
     """
     splines = dict()
+    gamma_precision = kwargs.get('gamma_precision', 'flarestack')
+    smoothing_order = kwargs.get('smoothing_order', 'flarestack')
+    gamma_support_points = get_gamma_support_points(precision=gamma_precision)
+
 
     for gamma in gamma_support_points:
         splines[gamma] = create_gamma_2d_ratio_spline(
-            exp, mc, sin_dec_bins, log_e_bins, gamma)
+            exp, mc, sin_dec_bins, log_e_bins, gamma, smoothing_order)
+
+    if not np.any(list(splines.values())):
+        logger.warning('No splines!')
+        return
 
     return splines
 
@@ -261,18 +354,21 @@ def create_bkg_spatial_spline(exp, sin_dec_bins):
     return bkg_spline
 
 
-def make_spline(seasons):
+def make_spline(seasons, **kwargs):
+    """Make the S/B splines for each season, as well as the background spline.
 
-    logging.info("Splines will be made to calculate the Signal/Background ratio of " \
-          "the MC to data. The MC will be weighted with a power law, for each" \
-          " gamma in: {0}".format(list(gamma_support_points)))
+    :param seasons: Seasons to iterate over
+    """
+
+    logger.info(f"Splines will be made to calculate the Signal/Background ratio of "
+                "the MC to data. The MC will be weighted with a power law, for each"
+                " gamma in: {list(get_gamma_support_points(**kwargs))}")
 
     for season in seasons.values():
-        SoB_path = SoB_spline_path(season)
-        make_individual_spline_set(season, SoB_path)
+        SoB_path = SoB_spline_path(season, **kwargs)
+        make_individual_spline_set(season, SoB_path, **kwargs)
         make_background_spline(season)
 
-
 def make_plot(hist, savepath, x_bins, y_bins, normed=True, log_min=5,
               label_x=r"$\sin(\delta)$", label_y="log(Energy)"):
     if normed:
@@ -299,9 +395,10 @@ def make_plot(hist, savepath, x_bins, y_bins, normed=True, log_min=5,
     plt.savefig(savepath)
     plt.close()
 
-def make_individual_spline_set(season, SoB_path):
+
+def make_individual_spline_set(season, SoB_path, **kwargs):
     try:
-        logging.info("Making splines for {0}".format(season.season_name))
+        logger.info("Making splines for {0}".format(season.season_name))
         # path = SoB_spline_path(season)
 
         exp = season.get_background_model()
@@ -310,9 +407,9 @@ def make_individual_spline_set(season, SoB_path):
         sin_dec_bins = season.sin_dec_bins
         log_e_bins = season.log_e_bins
 
-        splines = create_2d_splines(exp, mc, sin_dec_bins, log_e_bins)
+        splines = create_2d_splines(exp, mc, sin_dec_bins, log_e_bins, **kwargs)
 
-        logging.info("Saving to {0}".format(SoB_path))
+        logger.info("Saving to {0}".format(SoB_path))
 
         try:
             os.makedirs(os.path.dirname(SoB_path))
@@ -322,6 +419,9 @@ def make_individual_spline_set(season, SoB_path):
         with open(SoB_path, "wb") as f:
             Pickle.dump(splines, f)
 
+        if isinstance(splines, type(None)):
+            return
+
         base_plot_path = get_base_sob_plot_dir(season)
 
         exp_hist = create_bkg_2d_hist(exp, sin_dec_bins, log_e_bins)
@@ -329,7 +429,9 @@ def make_individual_spline_set(season, SoB_path):
         # Generate plots
         for gamma in np.linspace(1.0, 4.0, 7):
 
-            plot_path = base_plot_path + "gamma=" + str(gamma) + "/"
+            plot_path = base_plot_path + "gamma=" + str(gamma) + "/" + \
+                        f'precision{kwargs.get("gamma_precision", "flarestack")}_' \
+                        f'smoothing{kwargs.get("smoothing_order", "flarestack")}'
 
             try:
                 os.makedirs(plot_path)
@@ -351,7 +453,11 @@ def make_individual_spline_set(season, SoB_path):
             for s in sin_dec_bins:
                 z_line = []
                 for e in log_e_bins:
-                    z_line.append(splines[gamma](e, s)[0][0])
+                    # logging.debug(f'{e}, {s}')
+                    try:
+                        z_line.append(splines[gamma](e, s)[0][0])
+                    except:
+                        z_line.append(splines[gamma]((e, s)))
                 Z.append(z_line)
 
             Z = np.array(Z).T
@@ -386,7 +492,7 @@ def make_background_spline(season):
 
     bkg_spline = create_bkg_spatial_spline(bkg, sin_dec_bins)
 
-    logging.info("Saving to".format(bkg_path))
+    logger.info("Saving to".format(bkg_path))
     try:
         os.makedirs(os.path.dirname(bkg_path))
     except OSError:
@@ -411,13 +517,19 @@ def make_background_spline(season):
     plt.close()
 
 
-def load_spline(season):
-    path = SoB_spline_path(season)
+def load_spline(season, **kwargs):
+    path = SoB_spline_path(season, **kwargs)
 
-    logging.debug("Loading from {0}".format(path))
+    logger.debug(f"Loading from {path}")
 
-    with open(path, "rb") as f:
-        res = Pickle.load(f)
+    try:
+        with open(path, "rb") as f:
+            res = Pickle.load(f)
+    except FileNotFoundError:
+        logger.info(f"No cached spline found at {path}. Creating this file instead.")
+        make_individual_spline_set(season, path, **kwargs)
+        with open(path, "rb") as f:
+            res = Pickle.load(f)
 
     return res
 
@@ -425,9 +537,80 @@ def load_spline(season):
 def load_bkg_spatial_spline(season):
     path = bkg_spline_path(season)
 
-    logging.debug("Loading from {0}".format(path))
+    logger.debug(f"Loading from {path}")
 
-    with open(path, "rb") as f:
-        res = Pickle.load(f)
+    try:
+        with open(path, "rb") as f:
+            res = Pickle.load(f)
+    except FileNotFoundError:
+        logger.info(f"No cached spline found at {path}. Creating this file instead.")
+        make_background_spline(season)
+        with open(path, "rb") as f:
+            res = Pickle.load(f)
 
-    return res
\ No newline at end of file
+    return res
+
+
+# def delete_old_splines():
+#     """Deletes previously produced splines of the SoB energy PDF histogram, the spatial background PDF and the
+#     acceptance function"""
+#     logging.info('Deleting old splines!')
+#     directories_to_clear = [SoB_spline_dir, bkg_spline_dir, acc_f_dir]
+#     for d in directories_to_clear:
+#         logging.debug(f'clearing {d}')
+#         shutil.rmtree(d)
+#         os.mkdir(d)
+
+
+# def use_precision(mode='flarestack'):
+#     """
+#     Configures the current environment to use the desired precision in gamma.
+#     Deletes previously produced splines if precision changes.
+#     :param mode: float or 'flarestack' or 'skyLab', default:'flarestack'
+#     """
+#
+#     old_precision = os.environ.get(environment_precision_key, None)
+#     new_precision = '0.025' if mode in ['flaresatck', 'Flarestack', 'default'] else \
+#         '0.1' if (mode in ['SkyLab', 'skylab', 'skylab_splines']) or ('skylab_splines' in mode) else \
+#         mode if isinstance(mode, float) else \
+#         None
+#
+#     if not new_precision:
+#         logger.warning(f'Mode {mode} not known! Use "Flarestack", "SkyLab" or a float '
+#                        f'to specify the order of the interpolating spline.')
+#
+#     if not new_precision or (new_precision != old_precision):
+#         logger.info(f'Gamma precision has changed from {old_precision} to {new_precision}')
+#         delete_old_splines()
+#         os.environ[environment_precision_key] = str(new_precision)
+#     else:
+#         logging.info(f'New precision {new_precision} same as old precision {old_precision}.')
+#
+#     logging.info(f'Gamma precision is now {os.environ[environment_precision_key]}')
+
+
+# def use_smoothing(mode='flarestack'):
+#     """
+#     Configures the current environment to use the desired smoothing order when building energy PDFs
+#     Deletes previously produced splines if precision changes.
+#     :param mode: int or 'flarestack' or 'skyLab', default:'flarestack'
+#     """
+#
+#     old_smoothing_order = os.environ.get(environment_smoothing_key, None)
+#     new_smoothing_order = '2' if mode in ['default', 'flarestack', 'Flarestack'] else \
+#         '1' if mode in ['SkyLab', 'skylab', 'skylab_splines'] else \
+#         str(mode) if isinstance(mode, int) else \
+#         None
+#
+#     if not new_smoothing_order:
+#         logger.warning(f'Mode {mode} not known! Use "Flarestack", "SkyLab" or an integer '
+#                        f'to specify the order of the interpolating spline.')
+#
+#     if not old_smoothing_order or (old_smoothing_order != new_smoothing_order):
+#         logger.info(f'Smoothing order changed from {old_smoothing_order} to {new_smoothing_order}')
+#         delete_old_splines()
+#         os.environ[environment_smoothing_key] = str(new_smoothing_order)
+#     else:
+#         logging.info(f'New PDF smoothing order {new_smoothing_order} is the same as old one {old_smoothing_order}')
+#
+#     logging.info(f'Smoothing order is now {os.environ[environment_smoothing_key]}')
